#coding:utf8

'''
    JSON工具
'''
import ujson
from flask import escape

def loads(d):
    return ujson.loads(d)

def dumps(d, html_safe=False):
    if html_safe:
        d = html_escape(d)
    return ujson.dumps(d)

def html_escape(d):
    if type(d) == dict:
        for k in d:
            d[k] = html_escape(d[k])
    elif type(d) in [list, tuple]:
        for i in xrange(len(d)):
            d[i] = html_escape(d[i])
    elif type(d) in [int, float]:
        pass
    else:
        d = str(escape(d))
    return d


def json_success(**kwargs):
    kwargs['code'] = 0
    return kwargs



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author = "wudb@pv.cc"
__doc__ = \
"""**********************************************************
*************************************************************
**                                                         **
**        Copyright (C) pv.cc 2016 AllRights Reserved      **
**                                                         **
**    HISTORY:                                             **
**          V1.00       2016-11-09       Creation.         **
**                                                         **
*************************************************************

**********************************************************"""
import os, sys
import traceback
import ConfigParser

from flask import current_app

def get_key(identify, section, key):
    """
    @identify: 配置文件名称(不带key文件后缀), 例如 fund51.key 则传 fund51
    @section: 配置文件的配置项名称，例如fund51.key内的[fund51_prod]
    @key: 配置项的key名称，该函数将返回该key对应的value值
    为了避免频繁的磁盘io读取配置文件，将采用redis缓存缓存该key3600秒的时间
    """
    config_file = "keys/%s.key" % identify
    redis_key = 'keys-' + identify + section + key

    value = current_app.redis.get(redis_key)
    if value:
        return value

    cp = ConfigParser.ConfigParser()
    try:
        cp.read(config_file)
        value = cp.read(section, key)
    except Exception, e:
        raise traceback.format_exc()

    current_app.redis.set(redis_key, value)
    current_app.redis.expire(redis_key, 3600)

    return value



# coding:utf8

'''
    这个模块包含设置告警的代码
'''
import os
import sys
import random
import string
import logging
import logging.handlers
from flask import request, current_app

reload(sys)
sys.setdefaultencoding('utf8')


def get_ip_address(ifname='eth1'):
    import socket
    import fcntl
    import struct
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        return socket.inet_ntoa(fcntl.ioctl(
            s.fileno(),
            0x8915, # SIOCGIFADDR  
            struct.pack('256s', ifname[:15])
        )[20:24])
    except:
        return ''

ip = get_ip_address()

try:
    #server = os.environ["HOSTNAME"]
    import socket
    server = socket.gethostname()
except:
    server = "unkown"

class MySMTPHandler(logging.handlers.SMTPHandler):

    def emit(self, record):
        try:
            #r = random.randint(0,100)
            #if r <= 30:
            #    return
            import smtplib
            try:
                from email.utils import formatdate
            except ImportError:
                formatdate = self.date_time
            port = self.mailport
            if not port:
                port = smtplib.SMTP_PORT
            smtp = smtplib.SMTP(self.mailhost, port)
            real_ip = request.headers.get("X-Real-Ip", "")
            ref = request.headers.get("Referer", "")
            header = "From: %s(%s)\r\n\r\nX-Real-Ip:%s\r\nReferer:%s\r\n" % (server, ip, real_ip, ref)
            msg = header + "%s\r\n%s\r\n%s\r\n" % (ip, request.url, request.values.to_dict()) + self.format(record)
            msg = "From: %s\r\nTo: %s\r\nSubject: %s\r\nDate: %s\r\n\r\n%s" % (
                            self.fromaddr,
                            string.join(self.toaddrs, ","),
                            "%s (%s)" % (self.getSubject(record), server),
                            formatdate(), msg)
            if self.username:
                smtp.login(self.username, self.password)
            smtp.sendmail(self.fromaddr, self.toaddrs, msg)
            smtp.quit()
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


def enable_logging(app):
    config = app.config['LOGGING']
    config_mail = config.get('SMTP')
    if config_mail:  #如果存在smtp配置
        app.logger.info('Add SMTP Logging Handler')
        mail_handler = MySMTPHandler(
            config_mail['HOST'],  #smtp 服务器地址
            config_mail['USER'],  #smtp 发件人
            config_mail['TOADDRS'],  #smtp 收件人
            config_mail['SUBJECT'],  #smtp 主题
            (config_mail['USER'], config_mail['PASSWORD']))  #smtp账号密码
        mail_handler.setLevel(logging.ERROR)
        app.logger.addHandler(mail_handler)
    else:
        app.logger.info('No SMTP Config Found')

    config_file = config.get('FILE')
    if config_file:  #如果存在文件配置
        app.logger.info('Add File Logging Handler')
        file_handler = logging.handlers.RotatingFileHandler(
            config_file['PATH'],  #文件路径
            #但个文件大小 默认10M 
            maxBytes=config_file.setdefault('MAX_BYTES', 1024 * 1024 * 10),
            #文件备份>数量 默认5个
            backupCount=config_file.setdefault('BACKUP_COUNT', 5),
        )
        file_handler.setLevel(logging.INFO)
        formatter = logging.Formatter('[%(asctime)s | %(levelname)s | %(funcName)s] %(message)s')
        file_handler.setFormatter(formatter)
        app.logger.addHandler(file_handler)
    else:
        app.logger.info('No FILE Config Found')




#coding=utf-8

from functools import wraps, partial
from flask import abort, current_app, request, g


def get_param(key, default=None, type=None, required=False, mapping_key=None, \
                    minvalue=None, maxvalue=None, choices=None, autominvalue=False, automaxvalue=False):
    """获取request参数
    :param key: 参数名
    :param default: 参数默认值
    :param type: 参数类型
    :param mapping: request.XXXX (args, form, headers, values)
    :param required: 参数是否必须
    :param minvalue 限制参数最小值
    :param maxvalue 限制参数最大值
    :param choices 限制参数值选择
    :param autominvalue 值小于minvalue则调整为minvalue
    :param automaxvalue 值大于maxvalue则调整为maxvalue
    """
    mapping = getattr(request, mapping_key, {}) 
    if required is True and key not in mapping:
        print "missing param %s" % key 
        abort(400, "missing param %s" % key) 
    value = mapping.get(key, default)
    if type is not None:
        try:
            value = type(value)
        except ValueError, TypeError:
            print "invild param type %s, %s needed" % (key, type) 
            abort(400, "invild param type %s, %s needed" % (key, type)) 
        except Exception, e:
            print e
            abort(500)
    if minvalue is not None and value < minvalue:
        if autominvalue is True:
            value = minvalue
        else:
            abort(400, "param %s less than minvalue %s" % (key, minvalue))
    if maxvalue is not None and value > maxvalue:
        if automaxvalue is True:
            value = maxvalue
        else:
            abort(400, "param %s bigger than maxvalue %s" % (key, maxvalue))
    if choices is not None and value not in choices:
        abort(400, "prams %s is not available" % key)
    return value

request_args = partial(get_param, mapping_key="args")
request_form = partial(get_param, mapping_key="form")
request_values = partial(get_param, mapping_key="values")
request_headers = partial(get_param, mapping_key="headers")


def mysql_read_slave(func):
    @wraps(func)
    def decorate(*args, **kwargs):
        g.__bind = 'slave'
        return func(*args, **kwargs)
    return decorate



#coding:utf8

'''
    额外工具
'''
import struct
import socket
import hashlib

from flask import request, current_app, request, session
from qiniu import Auth, put_data 


def Ip2Int(ip):
    return struct.unpack("!I",socket.inet_aton(ip))[0]

def Int2Ip(i):
    return socket.inet_ntoa(struct.pack("!I",i))

def get_ip():
    if 'X-Forwarded-For' in request.headers:
        return request.headers['X-Forwarded-For'].split(',')[0]

    return request.headers.get('X-Real-Ip') or request.remote_addr

def upload_img(filename, data, bucket_name, config=None):
    if not config:
        config = current_app.config

    cfg = config['QINIU_SETTINGS']
    access_key = cfg['access_key']
    secret_key = cfg['secret_key']
    
    q = Auth(access_key, secret_key)
    token = q.upload_token(bucket_name, filename)
    ret, info = put_data(token, filename, data)
    
    assert ret['key'] == filename
    host = cfg['buckets'][bucket_name]
    return 'http://%s/%s'%(host, filename)

def hash_pwd(pwd):
    return hashlib.sha1(pwd).hexdigest()



#coding:utf-8
import bisect
import socket,struct
import json
from SimpleXMLRPCServer import SimpleXMLRPCServer
import SocketServer
from os.path import join, dirname

class ipCool(object):
    def __init__(self, path = None):
        if not path:
            path = join(dirname(__file__), 'ipcool')
        self.ipmap = {}
        self.iplist = []
        self.ipfile = open(path, 'r')
        self.readIps()
        self.ipfile.close()
        _tmp = json.load(open(join(dirname(__file__), 'provinces.json')))
        d = {}
        for row in _tmp['provinces']:
            _x = {}
            for _c in row['citys']:
                _x.update(_c)
            id = str(row['id'])
            d[id] = {
                'name': row['name'],
                'citys': _x
            }
        self.locations = d
        self.isps = {
            '1' : u'电信',
            '2' : u'联通',
            '3' : u'教育',
            '4' : u'移动',
            '5' : u'铁通',
            '6' : u'网通',
            '0' : u'',
         }

        
    def readIps(self):
        lines = self.ipfile.read().split('\n')
        for line in lines:
            if line == '' : continue
            nums = line.split('\t')
            key = int(nums[0].strip())
            self.iplist.append(key)
            self.ipmap[key] = nums[1:]
        self.iplist.sort()


    def targetIp(self, ipnum):
        ipnum = int(ipnum)
        pos = bisect.bisect_right(self.iplist, ipnum) - 1
        key = self.iplist[pos]
        if int(self.ipmap[key][0]) < ipnum:
            return ['0', '0', '0']
        return self.ipmap[key][1:] 
    
    def ipaton(self, ipstr):
        return struct.unpack('!I', socket.inet_aton(ipstr))[0]

_IC = ipCool()
def parse_ip(ip, detail = True):
    try:
        _tmp = _IC.ipaton(ip)
    except:
        return {'pro' : "", 'city' : "", 'isp' : ""}
    pro, city, isp = _IC.targetIp(_tmp)
    ret = {'pro' : pro, 'city' : city, 'isp' : isp}
    if detail:
        info = _IC.locations.get(pro, {})
        ret['pro_d'] = info.get('name', '')
        ret['city_d'] = info.get('citys', {}).get(city, '')
        ret['isp_d'] = _IC.isps.get(isp, '')
    return ret

def list_locations():
    return _IC.locations 

def list_isps():
    return _IC.isps

class ThreadingRPCServer(SocketServer.ThreadingMixIn, SimpleXMLRPCServer):
    pass

if __name__ == '__main__':
    print parse_ip('113.66.189.243')




# -*- coding:utf-8 -*-

import cfgtools
import flask_helper
import flask_logging
import json_utils
import utils
import readip
from .readip import (
    ipCool,
    ThreadingRPCServer,
)

__version__ = '0.0.1'
__all__ = ['cfgtools', 'flask_helper', 'flask_logging',
            'json_utils', 'utils', 'ipCool', 'ThreadingRPCServer', 'readip']




# -*- coding:utf-8 -*-

from setuptools import setup, find_packages

setup(
      name='feiwu',   #名称
      version='0.0.2',  #版本
      description="some useful module on flask developer", #描述
      keywords='web flask',
      author='chen',  #作者
      author_email='chenyehai737@qq.com', #作者邮箱
      license = 'MIT',
      url='', #作者链接
      packages=['feiwu'],
      package_data={
        'feiwu': ['ipcool', 'provinces.json'],
      },
      zip_safe=False,
      classifiers=[
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
      ],
      install_requires=[      #需求的第三方模块
        'Flask==0.10.1',
        'requests==2.7.0',
        'ujson==1.33',
        'qiniu==7.0.4',
      ],
      entry_points={
      },
)

